package base {
	

	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class ProjSpawner extends FlxObject {
		public var id:int;
		/*
		 * Status for being on and off
		 */
		private var _spawning:Boolean;
		/*
		 * Used for a one off spawn trigger
		 */
		private var _trigger:Boolean;
		/*
		 * Pool of projectile objects 
		 */
		private var _projectilePool:ObjectPool;
		/*
		 * Interval of time between firing bullets
		 */
		private var _interval:Number;
		/*
		 * Timer for determining if interval has passed
		 */
		private var _projectileTimer:Number;
		/*
		 * Array of FlxPoint locations in the form of offsets from which bullets spawn
		 */
		protected var _spawnLocations:Array;
		/*
		 * Array of FlxSprite objects which represent the visual component of the spawn locations
		 */
		private var _spawnSprites:Array;
		/*
		 * Determines whether projectiles cycle or fire all at once
		 */
		private var _cycle:Boolean;
		/*
		 * FlxObject which this Spawner is attached to and will follow it like a lost puppy
		 */
		protected var _parent:FlxObject;

		/*
		 * Creates a spawner for projectiles with a designated and initialized group for adding
		 * projectiles to.
		 * Also sets the interval for the spawner and optional follow target.
		 */
		public function ProjSpawner(X:Number, Y:Number, Interval:Number, ProjectileClass:Class, ProjectileQuantity:int, Parent:FlxObject = null) {
			super(X, Y);
			moves = false;
			solid = false;
			visible = false;
			
			_spawning = false;
			_trigger = false;
			
			_parent = Parent;
			
			_spawnLocations = [];
			_spawnSprites = [];
			
			// Creates the pool with objects of the appropriate type and quantity
			_projectilePool = new ObjectPool(ProjectileClass, ProjectileQuantity);
			
			_interval = Interval;
			
			// Don't start firing immediately
			_projectileTimer = 0.1;
			
			_cycle = false;
		}

		/*
		 * Add a location to the list of spawn locations in the form of an X, Y offset
		 * from the location of the spawner
		 */
		public function addLocation(X:Number, Y:Number, Sprite:FlxSprite = null):void {
			_spawnLocations.push(new FlxPoint(X, Y));
			_spawnSprites.push(Sprite);
		}

		/*
		 * Override this function and add any other necessary parameters
		 * Must call send after setting any necessary parameters in p
		 */
		public function sendProjectile(X:Number, Y:Number, p:Proj, locationIndex:int):void {
			p.send(X, Y, locationIndex);
		}

		/*
		 * Update ourselves based on the follow target
		 * Spawn bullets
		 * Update location spawn sprites
		 */
		override public function update():void {
			// Change our position to that of our follow object
			if(_parent) {
				x = _parent.x;
				y = _parent.y;
			}
			
			// Spawn if we are turned on
			if(_spawning || _trigger) {
				// Adjust Timer
				_projectileTimer -= FlxG.elapsed;
			
				var i:int;
				// Send the projectile
				if(_projectileTimer < 0 || _trigger) {
					var sll:int = _spawnLocations.length;
					
					if(_cycle) {
						// Location in the cycle is object_index % location_size
						i = _projectilePool.objectIndex % sll;
						sendProjectile(x + FlxPoint(_spawnLocations[i]).x, y + FlxPoint(_spawnLocations[i]).y, Proj(_projectilePool.getNextAvail()), i);
					} else {
						// Fire from all locations
						for (i = 0;i < sll;i++) {
							sendProjectile(x + FlxPoint(_spawnLocations[i]).x, y + FlxPoint(_spawnLocations[i]).y, Proj(_projectilePool.getNextAvail()), i);
						}
					}
					
					if(!_trigger) {
						_projectileTimer = _interval;
					} else {
						_trigger = false;
					}
					
				}
			}
			
			// Update spawn sprites
			var ssl:int = _spawnSprites.length;
			var ss:FlxSprite;
			for (i = 0;i < ssl;i++) {
				ss = FlxSprite(_spawnSprites[i]);
				if(ss) {
					ss.x = x + FlxPoint(_spawnLocations[i]).x - ss.origin.x;
					ss.y = y + FlxPoint(_spawnLocations[i]).y - ss.origin.y;
					ss.update();
				}
			}
			
			// Update ourselves
			super.update();
		}

		override public function render():void {
			// Render spawn sprites
			var i:int;
			var ssl:int = _spawnSprites.length;
			for (i = 0;i < ssl;i++) {
				if(_spawnSprites[i])
					FlxSprite(_spawnSprites[i]).render();
			}
			
			// There is no render for FlxObject but call it anyways for consistency
			super.render();
		}

		/*
		 * Start firing
		 */
		public function start():void {
			_spawning = true;
		}

		/*
		 * Stop firing
		 */
		public function stop():void {
			_spawning = false;
		}

		/*
		 * Trigger a single fire round
		 */
		public function trigger():void {
			_trigger = true;
		}

		/*
		 * Swap from spawning to not and vice versa
		 */
		public function swap():void {
			_spawning = !_spawning;
		}

		/*
		 * Interval of time between firing bullets
		 */
		public function get interval():Number {
			return _interval;
		}

		public function set interval(value:Number):void {
			_interval = value;
		}

		/*
		 * Whether or not this spawner cycles its locations
		 */
		public function set cycle(value:Boolean):void {
			_cycle = value;
		}

		/*
		 * Pool of projectile objects 
		 */
		public function get projectilePool():ObjectPool {
			return _projectilePool;
		}
	}
}
